#include "mapeditor.h"

#include <iostream>
#include <cstdlib>
#include <typeinfo>

#include <QObject>

#include "mainwindow.h"
#include "misc.h"
#include "warningdialog.h"


extern int statusbarTime;
extern Main *mainWindow;
extern QString tmpVymDir;
extern QString clipboardDir;
extern QString clipboardFile;
extern bool clipboardEmpty;
extern bool debug;
extern FlagRowObj *standardFlagsDefault;

extern QMenu* branchContextMenu;
extern QMenu* branchAddContextMenu;
extern QMenu* branchRemoveContextMenu;
extern QMenu* branchLinksContextMenu;
extern QMenu* branchXLinksContextMenuEdit;
extern QMenu* branchXLinksContextMenuFollow;
extern QMenu* floatimageContextMenu;
extern QMenu* canvasContextMenu;

extern Settings settings;
extern QString iconPath;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
MapEditor::MapEditor( VymModel *vm) 
{
	//cout << "Constructor ME "<<this<<endl;
	mapScene= new QGraphicsScene(NULL);
	mapScene->setBackgroundBrush (QBrush(Qt::white, Qt::SolidPattern));

	model=vm;
	model->setScene (mapScene);
	model->registerEditor(this);
//	model->addMapCenter();	//  FIXME create this in MapEditor until BO and MCO are independent of scene
	model->makeDefault();	// No changes in model so far

    setScene (mapScene);

    printer=NULL;

	// Create bitmap cursors, platform dependant
	HandOpenCursor=QCursor (QPixmap(iconPath+"cursorhandopen.png"),1,1);		
	PickColorCursor=QCursor ( QPixmap(iconPath+"cursorcolorpicker.png"), 5,27 ); 
	CopyCursor=QCursor ( QPixmap(iconPath+"cursorcopy.png"), 1,1 ); 
	XLinkCursor=QCursor ( QPixmap(iconPath+"cursorxlink.png"), 1,7 ); 

	setFocusPolicy (Qt::StrongFocus);

	pickingColor=false;
	drawingLink=false;
	copyingObj=false;

    editingBO=NULL;
    movingObj=NULL;

	printFrame=true;
	printFooter=true;

	setAcceptDrops (true);	

	//model->reposition();	//FIXME really still needed?


	// Action to embed LineEdit for heading in Scene
	editingHeading=false;
	lineEdit=new QLineEdit;
	lineEdit->hide();
	QGraphicsProxyWidget *pw=scene()->addWidget (lineEdit);
	pw->setZValue (100);

	QAction *a = new QAction( tr( "Edit heading","MapEditor" ), this);
	a->setShortcut ( Qt::Key_Return );					//Edit heading
	addAction (a);
    connect( a, SIGNAL( triggered() ), this, SLOT( editHeading() ) );
	a = new QAction( tr( "Edit heading","MapEditor" ), this);
	a->setShortcut ( Qt::Key_Enter);					//Edit heading
	addAction (a);
    connect( a, SIGNAL( triggered() ), this, SLOT( editHeading() ) );

	// Selections
	selectionColor =QColor (255,255,0);
	

	// Attributes	//FIXME testing only...
	QString k;
	AttributeDef *ad;
	attrTable= new AttributeTable();
	k="A - StringList";
	ad=attrTable->addKey (k,StringList);
	if (ad)
	{
		QStringList sl;
		sl <<"val 1"<<"val 2"<< "val 3";
		ad->setValue (QVariant (sl));
	}
	//attrTable->addValue ("Key A","P 1");
	//attrTable->addValue ("Key A","P 2");
	//attrTable->addValue ("Key A","P 3");
	//attrTable->addValue ("Key A","P 4");
	k="B - FreeString";
	ad=attrTable->addKey (k,FreeString);
	if (ad)
	{
		//attrTable->addValue ("Key B","w1");
		//attrTable->addValue ("Key B","w2");
	}
	k="C - UniqueString";
	ad=attrTable->addKey (k,UniqueString);
	if (ad)
	{
	//attrTable->addKey ("Key Prio");
	//attrTable->addValue ("Key Prio","Prio 1");
	//attrTable->addValue ("Key Prio","Prio 2");
	}
}

MapEditor::~MapEditor()
{
	//cout <<"Destructor MapEditor\n";
	// tmpMapDir is in tmpVymDir, so it gets removed automagically when vym closes
	
	//removeDir(QDir(tmpMapDir));	// FIXME check?!?
	model->unregisterEditor(this);
}

VymModel* MapEditor::getModel()
{
    return model;
}

QGraphicsScene * MapEditor::getScene()
{
    return mapScene;
}

void MapEditor::print()
{
	if ( !printer ) 
	{
		printer = new QPrinter;
		printer->setColorMode (QPrinter::Color);
		printer->setPrinterName (settings.value("/mainwindow/printerName",printer->printerName()).toString());
		printer->setOutputFormat((QPrinter::OutputFormat)settings.value("/mainwindow/printerFormat",printer->outputFormat()).toInt());
		printer->setOutputFileName(settings.value("/mainwindow/printerFileName",printer->outputFileName()).toString());
	}

	QRectF totalBBox=model->getTotalBBox();

	// Try to set orientation automagically
	// Note: Interpretation of generated postscript is amibiguous, if 
	// there are problems with landscape mode, see
	// http://sdb.suse.de/de/sdb/html/jsmeix_print-cups-landscape-81.html

	if (totalBBox.width()>totalBBox.height())
		// recommend landscape
		printer->setOrientation (QPrinter::Landscape);
	else	
		// recommend portrait
		printer->setOrientation (QPrinter::Portrait);

	if ( printer->setup(this) ) 
	// returns false, if printing is canceled
	{
		QPainter pp(printer);

		pp.setRenderHint(QPainter::Antialiasing,true);

		// Don't print the visualisation of selection
		model->unselect();

		QRectF mapRect=totalBBox;
		QGraphicsRectItem *frame=NULL;

		if (printFrame) 
		{
			// Print frame around map
			mapRect.setRect (totalBBox.x()-10, totalBBox.y()-10, 
				totalBBox.width()+20, totalBBox.height()+20);
			frame=mapScene->addRect (mapRect, QPen(Qt::black),QBrush(Qt::NoBrush));
			frame->setZValue(0);
			frame->show();    
		}		


		double paperAspect = (double)printer->width()   / (double)printer->height();
		double   mapAspect = (double)mapRect.width() / (double)mapRect.height();
		int viewBottom;
		if (mapAspect>=paperAspect)
		{
			// Fit horizontally to paper width
			//pp.setViewport(0,0, printer->width(),(int)(printer->width()/mapAspect) );	
			viewBottom=(int)(printer->width()/mapAspect);	
		}	else
		{
			// Fit vertically to paper height
			//pp.setViewport(0,0,(int)(printer->height()*mapAspect),printer->height());	
			viewBottom=printer->height();	
		}	
		
		if (printFooter) 
		{
			// Print footer below map
			QFont font;		
			font.setPointSize(10);
			pp.setFont (font);
			QRectF footerBox(0,viewBottom,printer->width(),15);
			// FIXME fileName not any longer available here: pp.drawText ( footerBox,Qt::AlignLeft,"VYM - " +fileName);
			pp.drawText ( footerBox, Qt::AlignRight, QDate::currentDate().toString(Qt::TextDate));
		}
		mapScene->render (
			&pp, 
			QRectF (0,0,printer->width(),printer->height()-15),
			QRectF(mapRect.x(),mapRect.y(),mapRect.width(),mapRect.height())
		);
		
		// Viewport has paper dimension
		if (frame)  delete (frame);

		// Restore selection
		model->reselect();

		// Save settings in vymrc
		settings.writeEntry("/mainwindow/printerName",printer->printerName());
		settings.writeEntry("/mainwindow/printerFormat",printer->outputFormat());
		settings.writeEntry("/mainwindow/printerFileName",printer->outputFileName());
	}
}

void MapEditor::setAntiAlias (bool b)
{
	setRenderHint(QPainter::Antialiasing,b);
}

void MapEditor::setSmoothPixmap(bool b)
{
	setRenderHint(QPainter::SmoothPixmapTransform,b);
}

void MapEditor::toggleStandardFlag(QString f)
{
	BranchObj *bo=model->getSelectedBranch();
	if (bo) 
	{
		QString u,r;
		if (bo->isSetStandardFlag(f))
		{
			r="unsetFlag";
			u="setFlag";
		}	
		else
		{
			u="unsetFlag";
			r="setFlag";
		}	
		model->saveState(
			bo,
			QString("%1 (\"%2\")").arg(u).arg(f), 
			bo,
			QString("%1 (\"%2\")").arg(r).arg(f),
			QString("Toggling standard flag \"%1\" of %2").arg(f).arg(model->getObjectName(bo)));
		bo->toggleStandardFlag (f,mainWindow->useFlagGroups());
		model->updateSelection();	// geometry has changed
	}
}

AttributeTable* MapEditor::attributeTable()
{
	return attrTable;
}

void MapEditor::testFunction1()
{
	//BranchObj *bo=model->getSelectedBranch();
	//if (bo) model->moveAway (bo);
	//if (bo) bo->setLinkStyle (LinkableMapObj::Line);
	

	// Displacement and animation of all non-mainbranches
	QPointF p;
	QPointF q;
	BranchObj *bo;
	bo=model->first();
	while (bo) 
	{
		if (bo->getDepth() >0 && !bo->hasScrolledParent(bo) )
		{
			p=QPointF (qrand() %600-300, qrand () %600-300);
			bo->setRelPos();
			q=bo->getRelPos();
			model->startAnimation (bo,p, q);
		}
		bo=model->next(bo);
	}



/* TODO Hide hidden stuff temporary, maybe add this as regular function somewhere
	if (hidemode==HideNone)
	{
		setHideTmpMode (HideExport);
		mapCenter->calcBBoxSizeWithChilds();
		QRectF totalBBox=mapCenter->getTotalBBox();
		QRectF mapRect=totalBBox;
		QCanvasRectangle *frame=NULL;

		cout << "  map has =("<<totalBBox.x()<<","<<totalBBox.y()<<","<<totalBBox.width()<<","<<totalBBox.height()<<")\n";
	
		mapRect.setRect (totalBBox.x(), totalBBox.y(), 
			totalBBox.width(), totalBBox.height());
		frame=new QCanvasRectangle (mapRect,mapScene);
		frame->setBrush (QColor(white));
		frame->setPen (QColor(black));
		frame->setZValue(0);
		frame->show();    
	}	
	else	
	{
		setHideTmpMode (HideNone);
	}	
	cout <<"  hidemode="<<hidemode<<endl;
	*/
}	
	
void MapEditor::testFunction2()
{

/*
	// Toggle hidemode
	if (hidemode==HideExport)
		setHideTmpMode (HideNone);
	else	
		setHideTmpMode (HideExport);
*/		
}

void MapEditor::editHeading()
{
	if (editingHeading)
	{
		editHeadingFinished();
		return;
	}
	BranchObj *bo=model->getSelectedBranch();
	if (bo)
	{
		model->setSelectionBlocked(true);

		lineEdit->setText (bo->getHeading());
		QPoint p = mapTo (this,bo->getAbsPos().toPoint() );
		lineEdit->setGeometry(p.x(),p.y(),230,25);
		lineEdit->selectAll();
		lineEdit->show();
		lineEdit->setFocus();
		lineEdit->grabKeyboard();
		editingHeading=true;
	}

}
void MapEditor::editHeadingFinished()
{
	editingHeading=false;
	lineEdit->releaseKeyboard();
	model->setHeading (lineEdit->text() );
	model->setSelectionBlocked(false);
	lineEdit->hide();

	// Maybe reselect previous branch 
	mainWindow->editHeadingFinished (model);
}


void MapEditor::contextMenuEvent ( QContextMenuEvent * e )
{
	// Lineedits are already closed by preceding
	// mouseEvent, we don't need to close here.

    QPointF p = mapToScene(e->pos());
    LinkableMapObj* lmo=model->findMapObj(p, NULL);
	
    if (lmo) 
	{	// MapObj was found
		if (model->getSelectedLMO() != lmo)
		{
			// select the MapObj
			model->select(lmo);
		}
		// Context Menu 
		if (model->getSelectedBranch() ) 
		{
			// Context Menu on branch or mapcenter
			model->updateActions();
			branchContextMenu->popup(e->globalPos() );
		} else
		{
			if (model->getSelectedFloatImage() )
			{
				// Context Menu on floatimage
				model->updateActions();
				floatimageContextMenu->popup(e->globalPos() );
			}	
		}	
	} else 
	{ // No MapObj found, we are on the Canvas itself
		// Context Menu on scene
		model->updateActions();
		
		// Open context menu synchronously to position new mapcenter
		model->setContextPos (p);
		canvasContextMenu->exec(e->globalPos() );
		model->unsetContextPos ();
    } 
	e->accept();
}

void MapEditor::keyPressEvent(QKeyEvent* e)
{
	if (e->modifiers() & Qt::ControlModifier)
	{
		switch (mainWindow->getModMode())
		{
			case Main::ModModeColor: 
				setCursor (PickColorCursor);
				break;
			case Main::ModModeCopy: 
				setCursor (CopyCursor);
				break;
			case Main::ModModeXLink: 
				setCursor (XLinkCursor);
				break;
			default :
				setCursor (Qt::ArrowCursor);
				break;
		} 
	}	
}

void MapEditor::keyReleaseEvent(QKeyEvent* e)
{
	if (!(e->modifiers() & Qt::ControlModifier))
		setCursor (Qt::ArrowCursor);
}

void MapEditor::mousePressEvent(QMouseEvent* e)
{
	// Ignore right clicks, these will go to context menus
	if (e->button() == Qt::RightButton )
	{
		e->ignore();
		return;
	}

	//Ignore clicks while editing heading
	if (model->isSelectionBlocked() ) 
	{
		e->ignore();
		return;
	}

    QPointF p = mapToScene(e->pos());
    LinkableMapObj* lmo=model->findMapObj(p, NULL);
	
	e->accept();

	//Take care of  system flags _or_ modifier modes
	//
	if (lmo && (typeid(*lmo)==typeid(BranchObj) ||
		typeid(*lmo)==typeid(MapCenterObj) ))
	{
		QString foname=((BranchObj*)lmo)->getSystemFlagName(p);
		if (!foname.isEmpty())
		{
			// systemFlag clicked
			model->selectInt (lmo);
			if (foname=="url") 
			{
				if (e->state() & Qt::ControlModifier)
					mainWindow->editOpenURLTab();
				else	
					mainWindow->editOpenURL();
			}	
			else if (foname=="vymLink")
			{
				mainWindow->editOpenVymLink();
				// tabWidget may change, better return now
				// before segfaulting...
			} else if (foname=="note")
				mainWindow->windowToggleNoteEditor();
			else if (foname=="hideInExport")		
				model->toggleHideExport();
			// FIXME needed? xelection.update();	
			return;	
		} 
	}	
	// No system flag clicked, take care of modmodes (CTRL-Click)
	if (e->state() & Qt::ControlModifier)
	{
		if (mainWindow->getModMode()==Main::ModModeColor)
		{
				pickingColor=true;
				setCursor (PickColorCursor);
				return;
		} 
		if (mainWindow->getModMode()==Main::ModModeXLink)
		{	
			BranchObj *bo_begin=NULL;
			if (lmo)
				bo_begin=(BranchObj*)(lmo);
			else	
				bo_begin=model->getSelectedBranch();
			if (bo_begin)	
			{
				drawingLink=true;
				linkingObj_src=bo_begin;
				tmpXLink=new XLinkObj (mapScene);
				tmpXLink->setBegin (bo_begin);
				tmpXLink->setEnd   (p);
				tmpXLink->setColor(model->getMapDefXLinkColor());
				tmpXLink->setWidth(model->getMapDefXLinkWidth());
				tmpXLink->updateXLink();
				tmpXLink->setVisibility (true);
				return;
			} 
		}
	}	// End of modmodes

    if (lmo) 
	{	
		cout << "ME::mouse pressed\n";
		cout << "  lmo="<<lmo<<endl;
		cout << "  h="<<((BranchObj*)lmo)->getHeading().toStdString()<<endl;
		// Select the clicked object

		// FIXME VM better let "find" return an index instead of lmo...
		// Get index of clicked LMO
		TreeItem *ti=lmo->getTreeItem();
		cout << "  lmo="<<lmo<<"    lmo(ti)="<<ti->getLMO()<<endl;
		cout << "  ti ("<<ti->row()<<","<<ti->column()<<") = "<<ti<<endl;
		//QModelIndex ix=model->index( ti->row(), ti->column(), model->index (0,0,QModelIndex()) );
		QModelIndex ix=model->index(ti);
		model->getSelectionModel()->select (ix,QItemSelectionModel::ClearAndSelect  );

		// Left Button	    Move Branches
		if (e->button() == Qt::LeftButton )
		{
			//movingObj_start.setX( p.x() - selection->x() );// TODO replaced selection->lmo here	
			//movingObj_start.setY( p.y() - selection->y() );	
			movingObj_start.setX( p.x() - lmo->x() );	
			movingObj_start.setY( p.y() - lmo->y() );	
			movingObj_orgPos.setX (lmo->x() );
			movingObj_orgPos.setY (lmo->y() );
			lmo->setRelPos();
			movingObj_orgRelPos=lmo->getRelPos();

			// If modMode==copy, then we want to "move" the _new_ object around
			// then we need the offset from p to the _old_ selection, because of tmp
			if (mainWindow->getModMode()==Main::ModModeCopy &&
				e->state() & Qt::ControlModifier)
			{
				BranchObj *bo=model->getSelectedBranch();
				if (bo)
				{
					copyingObj=true;
					bo->addBranch (model->getSelectedBranch());
					model->unselect();
					model->select(bo->getLastBranch());
					model->reposition();
				}
			} 

			movingObj=model->getSelectedLMO();	
		} else
			// Middle Button    Toggle Scroll
			// (On Mac OS X this won't work, but we still have 
			// a button in the toolbar)
			if (e->button() == Qt::MidButton )
				model->toggleScroll();
		model->updateActions();
		// FIXME needed? xelection.update();
	} else 
	{ // No MapObj found, we are on the scene itself
		// Left Button	    move Pos of sceneView
		if (e->button() == Qt::LeftButton )
		{
			movingObj=NULL;	// move Content not Obj
			movingObj_start=e->globalPos();
			movingCont_start=QPointF (
				horizontalScrollBar()->value(),
				verticalScrollBar()->value());
			movingVec=QPointF(0,0);
			setCursor(HandOpenCursor);
		} 
    } 
}

void MapEditor::mouseMoveEvent(QMouseEvent* e)
{
    QPointF p = mapToScene(e->pos());
	LinkableMapObj *lmosel=model->getSelectedLMO();

    // Move the selected MapObj
    if ( lmosel && movingObj) 
    {	
		// reset cursor if we are moving and don't copy
		if (mainWindow->getModMode()!=Main::ModModeCopy)
			setCursor (Qt::ArrowCursor);

		// To avoid jumping of the sceneView, only 
		// ensureSelectionVisible, if not tmp linked
		if (!lmosel->hasParObjTmp())
			model->ensureSelectionVisible ();
		
		// Now move the selection, but add relative position 
		// (movingObj_start) where selection was chosen with 
		// mousepointer. (This avoids flickering resp. jumping 
		// of selection back to absPos)
		
		// Check if we could link 
		LinkableMapObj* lmo=model->findMapObj(p, lmosel);
		

		FloatObj *fio=model->getSelectedFloatImage();
		if (fio)
		{
			fio->move   (p.x() -movingObj_start.x(), p.y()-movingObj_start.y() );		
			fio->setRelPos();
			fio->updateLink(); //no need for reposition, if we update link here
			model->updateSelection();	// position has changed

			// Relink float to new mapcenter or branch, if shift is pressed	
			// Only relink, if selection really has a new parent
			if ( (e->modifiers()==Qt::ShiftModifier) && lmo &&
				( (typeid(*lmo)==typeid(BranchObj)) ||
				  (typeid(*lmo)==typeid(MapCenterObj)) ) &&
				( lmo != fio->getParObj())  
				)
			{
				if (typeid(*fio) == typeid(FloatImageObj) && 
				( (typeid(*lmo)==typeid(BranchObj) ||
				  typeid(*lmo)==typeid(MapCenterObj)) ))  
				{

					// Also save the move which was done so far
					QString pold=qpointfToString(movingObj_orgRelPos);
					QString pnow=qpointfToString(fio->getRelPos());
					model->saveState(
						fio,
						"moveRel "+pold,
						fio,
						"moveRel "+pnow,
						QString("Move %1 to relative position %2").arg(model->getObjectName(fio)).arg(pnow));
					fio->getParObj()->requestReposition();
					model->reposition();

					model->linkFloatImageTo (model->getSelectString(lmo));
					//movingObj=lmosel;
					//movingObj_orgRelPos=lmosel->getRelPos();	

					model->reposition();
				}	
			}
		} else	
		{	// selection != a FloatObj
			if (lmosel->getDepth()==0)
			{
				// Move MapCenter
				if (e->buttons()== Qt::LeftButton && e->modifiers()==Qt::ShiftModifier) 
					((MapCenterObj*)lmosel)->moveAll(p.x() -movingObj_start.x(), p.y()-movingObj_start.y() );		
				else	
					lmosel->move   (p.x() -movingObj_start.x(), p.y()-movingObj_start.y() );		
				model->updateRelPositions();
			} else
			{	
				if (lmosel->getDepth()==1)
				{
					// Move mainbranch
					lmosel->move(p.x() -movingObj_start.x(), p.y()-movingObj_start.y() );		
					lmosel->setRelPos();
				} else
				{
					// Move ordinary branch
					if (lmosel->getOrientation() == LinkableMapObj::LeftOfCenter)
						// Add width of bbox here, otherwise alignRelTo will cause jumping around
						lmosel->move(p.x() -movingObj_start.x() , //lmosel->getBBox().width(), 
							p.y()-movingObj_start.y() +lmosel->getTopPad() );		
					else	
						lmosel->move(p.x() -movingObj_start.x(), p.y()-movingObj_start.y() -lmosel->getTopPad());
					lmosel->setRelPos();	
				} 

				// Maybe we can relink temporary?
				if (lmo && (lmo!=lmosel) && model->getSelectedBranch() && 
					 (typeid(*lmo)==typeid(BranchObj) ||
					  typeid(*lmo)==typeid(MapCenterObj)) ) 

				{
					if (e->modifiers()==Qt::ControlModifier)
					{
						// Special case: CTRL to link below lmo
						lmosel->setParObjTmp (lmo,p,+1);
					}
					else if (e->modifiers()==Qt::ShiftModifier)
						lmosel->setParObjTmp (lmo,p,-1);
					else
						lmosel->setParObjTmp (lmo,p,0);
				} else	
				{
					lmosel->unsetParObjTmp();
				}		
				// reposition subbranch
				lmosel->reposition();	
			} // depth>0

			QItemSelection sel=model->getSelectionModel()->selection();
			updateSelection(sel,sel);	// position has changed

		} // no FloatImageObj

		scene()->update();
		return;
	} // selection && moving_obj
		
	// Draw a link from one branch to another
	if (drawingLink)
	{
		 tmpXLink->setEnd (p);
		 tmpXLink->updateXLink();
	}	 
	
    // Move sceneView 
    if (!movingObj && !pickingColor &&!drawingLink && e->buttons() == Qt::LeftButton ) 
	{
		QPointF p=e->globalPos();
		movingVec.setX(-p.x() + movingObj_start.x() );
		movingVec.setY(-p.y() + movingObj_start.y() );
		horizontalScrollBar()->setSliderPosition((int)( movingCont_start.x()+movingVec.x() ));
		verticalScrollBar()->setSliderPosition((int)( movingCont_start.y()+movingVec.y() ) );
    }
}


void MapEditor::mouseReleaseEvent(QMouseEvent* e)
{
    QPointF p = mapToScene(e->pos());
	LinkableMapObj *dst;
	LinkableMapObj *lmosel=model->getSelectedLMO();
	// Have we been picking color?
	if (pickingColor)
	{
		pickingColor=false;
		setCursor (Qt::ArrowCursor);
		// Check if we are over another branch
		dst=model->findMapObj(p, NULL);
		if (dst && lmosel) 
		{	
			if (e->state() & Qt::ShiftModifier)
				model->colorBranch (((BranchObj*)dst)->getColor());
			else	
				model->colorSubtree (((BranchObj*)dst)->getColor());
		} 
		return;
	}

	// Have we been drawing a link?
	if (drawingLink)	
	{
		drawingLink=false;
		// Check if we are over another branch
		dst=model->findMapObj(p, NULL);
		if (dst && lmosel) 
		{	
			tmpXLink->setEnd ( ((BranchObj*)(dst)) );
			tmpXLink->updateXLink();
			tmpXLink->activate(); //FIXME savestate missing
			//model->saveStateComplete(QString("Activate xLink from %1 to %2").arg(model->getObjectName(tmpXLink->getBegin())).arg(model->getObjectName(tmpXLink->getEnd())) );	
		} else
		{
			delete(tmpXLink);
			tmpXLink=NULL;
		}
		return;
	}
	
    // Have we been moving something?
    if ( lmosel && movingObj ) 
    {	
		FloatImageObj *fo=model->getSelectedFloatImage();
		if(fo)
		{
			// Moved FloatObj. Maybe we need to reposition
		    QString pold=qpointfToString(movingObj_orgRelPos);
		    QString pnow=qpointfToString(fo->getRelPos());
			model->saveState(
				fo,
				"moveRel "+pold,
				fo,
				"moveRel "+pnow,
				QString("Move %1 to relative position %2").arg(model->getObjectName(fo)).arg(pnow));

			fo->getParObj()->requestReposition();
			model->reposition();
		}	

		// Check if we are over another branch, but ignore 
		// any found LMOs, which are FloatObjs
		dst=model->findMapObj(mapToScene(e->pos() ), lmosel);

		if (dst && (typeid(*dst)!=typeid(BranchObj) && typeid(*dst)!=typeid(MapCenterObj))) 
			dst=NULL;
		
		BranchObj *bo=model->getSelectedBranch();
		if (bo && bo->getDepth()==0)
		{	
            if (movingObj_orgPos != bo->getAbsPos())
            {
                QString pold=qpointfToString(movingObj_orgPos);
                QString pnow=qpointfToString(bo->getAbsPos());
                model->saveState(
                    bo,
                    "move "+pold,
                    bo,
                    "move "+pnow,
                    QString("Move mapcenter %1 to position %2").arg(model->getObjectName(bo)).arg(pnow));
            }
		}
	
		if (model->selectionType() == TreeItem::Branch )
		{	// A branch was moved
			
			// save the position in case we link to mapcenter
			QPointF savePos=QPointF (lmosel->getAbsPos()  );

			// Reset the temporary drawn link to the original one
			lmosel->unsetParObjTmp();

			// For Redo we may need to save original selection
			QString preSelStr=model->getSelectString(lmosel);

			copyingObj=false;	
			if (dst ) 
			{
				// We have a destination, relink to that

				BranchObj* bsel=model->getSelectedBranch();
				BranchObj* bdst=(BranchObj*)dst;

				QString preParStr=model->getSelectString (bsel->getParObj());
				QString preNum=QString::number (bsel->getNum(),10);
				QString preDstParStr;

				if (e->state() & Qt::ShiftModifier && dst->getParObj())
				{	// Link above dst
					preDstParStr=model->getSelectString (dst->getParObj());
					bsel->linkTo ( (BranchObj*)(bdst->getParObj()), bdst->getNum());
				} else 
				if (e->state() & Qt::ControlModifier && dst->getParObj())
				{
					// Link below dst
					preDstParStr=model->getSelectString (dst->getParObj());
					bsel->linkTo ( (BranchObj*)(bdst->getParObj()), bdst->getNum()+1);
				} else	
				{	// Append to dst
					preDstParStr=model->getSelectString(dst);
					bsel->linkTo (bdst,-1);
					if (dst->getDepth()==0) bsel->move (savePos);
				} 
				QString postSelStr=model->getSelectString(lmosel);
				QString postNum=QString::number (bsel->getNum(),10);

				QString undoCom="linkTo (\""+ 
					preParStr+ "\"," + preNum  +"," + 
					QString ("%1,%2").arg(movingObj_orgPos.x()).arg(movingObj_orgPos.y())+ ")";

				QString redoCom="linkTo (\""+ 
					preDstParStr + "\"," + postNum + "," +
					QString ("%1,%2").arg(savePos.x()).arg(savePos.y())+ ")";

				model->saveState (
					postSelStr,undoCom,
					preSelStr, redoCom,
					QString("Relink %1 to %2").arg(model->getObjectName(bsel)).arg(model->getObjectName(dst)) );

				model->reposition();	// not necessary if we undo temporary move  below
			} else
			{
				// No destination, undo  temporary move

				if (lmosel->getDepth()==1)
				{
					// The select string might be different _after_ moving around.
					// Therefor reposition and then use string of old selection, too
					model->reposition();

                    QPointF rp(lmosel->getRelPos());
                    if (rp != movingObj_orgRelPos)
                    {
                        QString ps=qpointfToString(rp);
                        model->saveState(
                            model->getSelectString(lmosel), "moveRel "+qpointfToString(movingObj_orgRelPos), 
                            preSelStr, "moveRel "+ps, 
                            QString("Move %1 to relative position %2").arg(model->getObjectName(lmosel)).arg(ps));
                    }
				}

				// Draw the original link, before selection was moved around
				if (settings.value("/animation/use",false).toBool() && lmosel->getDepth()>1) 
				{
					lmosel->setRelPos();	// calc relPos first for starting point
					QPointF dst=bo->getParObj()->getChildPos();
			//		if (lmosel->getOrientation()==LinkableMapObj::LeftOfCenter) dst.setX (dst.x()+lmosel->width() );
					
					model->startAnimation(
						(BranchObj*)lmosel,
						lmosel->getRelPos(),
						movingObj_orgRelPos
//						QPointF (movingObj_orgPos.x() - dst.x(), movingObj_orgPos.y() - dst.y() )
					);	
				} else	
					model->reposition();
			}
		}
		 model->updateSelection();
		// Finally resize scene, if needed
		scene()->update();
		movingObj=NULL;		

		// Just make sure, that actions are still ok,e.g. the move branch up/down buttons...
		model->updateActions();
	} else 
		// maybe we moved View: set old cursor
		setCursor (Qt::ArrowCursor);
    
}

void MapEditor::mouseDoubleClickEvent(QMouseEvent* e)
{
	if (model->isSelectionBlocked() ) 
	{
		e->ignore();
		return;
	}

	if (e->button() == Qt::LeftButton )
	{
		QPointF p = mapToScene(e->pos());
		LinkableMapObj *lmo=model->findMapObj(p, NULL);
		if (lmo) {	// MapObj was found
			// First select the MapObj than edit heading
			model->select (lmo);
			editHeading();
		}
	}
}

void MapEditor::resizeEvent (QResizeEvent* e)
{
	QGraphicsView::resizeEvent( e );
}

void MapEditor::dragEnterEvent(QDragEnterEvent *event)
{
	//for (unsigned int i=0;event->format(i);i++) // Debug mime type
	//	cerr << event->format(i) << endl;

	if (event->mimeData()->hasImage())
		event->acceptProposedAction();
	else	
		if (event->mimeData()->hasUrls())
			event->acceptProposedAction();
}

void MapEditor::dragMoveEvent(QDragMoveEvent *)
{
}

void MapEditor::dragLeaveEvent(QDragLeaveEvent *event)
{
	event->accept();
}

void MapEditor::dropEvent(QDropEvent *event)
{
	BranchObj *sel=model->getSelectedBranch();
	if (sel)
	{
		if (debug)
			foreach (QString format,event->mimeData()->formats()) 
				cout << "MapEditor: Dropped format: "<<qPrintable (format)<<endl;


		QList <QUrl> uris;
		if (event->mimeData()->hasImage()) 
		{
			 QVariant imageData = event->mimeData()->imageData();
			 model->addFloatImage (qvariant_cast<QPixmap>(imageData));
		} else
		if (event->mimeData()->hasUrls())
			uris=event->mimeData()->urls();

		if (uris.count()>0)
		{
			QStringList files;
			QString s;
			QString heading;
			BranchObj *bo;
			for (int i=0; i<uris.count();i++)
			{
				// Workaround to avoid adding empty branches
				if (!uris.at(i).toString().isEmpty())
				{
					bo=sel->addBranch();
					if (bo)
					{
						s=uris.at(i).toLocalFile();
						if (!s.isEmpty()) 
						{
						   QString file = QDir::fromNativeSeparators(s);
						   heading = QFileInfo(file).baseName();
						   files.append(file);
						   if (file.endsWith(".vym", false))
							   bo->setVymLink(file);
						   else
							   bo->setURL(uris.at(i).toString());
					   } else 
					   {
						   bo->setURL(uris.at(i).toString());
					   }

					   if (!heading.isEmpty())
						   bo->setHeading(heading);
					   else
						   bo->setHeading(uris.at(i).toString());
					}
				}
			}
			model->reposition();
		}
	}	
	event->acceptProposedAction();
}

void MapEditor::updateSelection(const QItemSelection &newsel,const QItemSelection &)
{
	// Reduce rectangles
	while (newsel.indexes().count() < selboxList.count() )
		delete selboxList.takeFirst();

	// Add additonal rectangles
	QGraphicsRectItem *sb;
	while (newsel.indexes().count() > selboxList.count() )
	{
		sb = mapScene->addRect(
			QRectF(0,0,0,0), 
			QPen(selectionColor),
			selectionColor);
		sb->setZValue(Z_SELBOX);
		sb->show();
		selboxList.append (sb);
	}

	// Reposition rectangles
	int i=0;
	QRectF bbox;
	QModelIndex index;

	TreeItem *ti;
	LinkableMapObj *lmo;
	foreach (sb,selboxList)
	{
		index=newsel.indexes().at(i);
		ti= static_cast<TreeItem*>(index.internalPointer());
		lmo=ti->getLMO();
		bbox=lmo->getBBox();
		sb->setRect (
			bbox.x(),bbox.y(), 
			bbox.width(), bbox.height());
		sb->setPen (selectionColor);	
		sb->setBrush (selectionColor);	
		i++;
	}
}

void MapEditor::updateCurrent (const QModelIndex &,const QModelIndex &)	//FIXME not used?
{

/* FIXME testing

	cout << "ME::updateCurrent\n";

	TreeItem *item = static_cast<TreeItem*>(newsel.internalPointer());
	LinkableMapObj *lmo=item->getLMO();
	cout << "  lmo="<<lmo<<endl;
	cout << "  h="<<((BranchObj*)lmo)->getHeading().toStdString()<<endl;
	*/

}

void MapEditor::setSelectionColor (QColor col)
{
	selectionColor=col;
	QItemSelection sel=model->getSelectionModel()->selection();
	updateSelection(sel,sel);
}

QColor MapEditor::getSelectionColor ()
{
	return selectionColor;
}


