#include "vymview.h"

#include <iostream>

#include "mainwindow.h"
#include "mapeditor.h"

extern Main *mainWindow;


VymView::VymView(VymModel *model)
{
	// Create TreeView
	treeview=new QTreeView;
	treeview->setModel ((QAbstractItemModel*)model);
	treeview->setMinimumWidth (350);
	treeview->setColumnWidth (0,350);

	selModel=treeview->selectionModel();
	model->setSelectionModel (selModel);
	connect (
		selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		this,SLOT (changeSelection(const QItemSelection &,const QItemSelection &)));

	// Create good old MapEditor
	MapEditor* me=model->getMapEditor();
	if (!me) me=new MapEditor (model);
	connect (
		selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		me,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));
	connect (
		selModel, SIGNAL (currentChanged(const QModelIndex &, const QModelIndex &)), 
		me,SLOT (updateCurrent(const QModelIndex &,const QModelIndex &)));

	// VymModel may want to update selection, e.g. during animation
	connect (
		model, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
		me,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));

	//me->viewport()->setFocus();	//FIXME needed?
	me->setAntiAlias (mainWindow->isAliased());
	me->setSmoothPixmap(mainWindow->hasSmoothPixmapTransform());

	addWidget (treeview);
	addWidget (me);

	// Set geometry
	QList <int> sizes;
	sizes.append (120);
	sizes.append (600);
	setSizes (sizes);
}

QItemSelectionModel* VymView::selectionModel() 
{
	if (treeview) 
		return selModel;
	else 
		std::cout <<"VymView::selectionModel: hey, no treeview so far???\n";
	return NULL;
}


void VymView::changeSelection (const QItemSelection &newSel, const QItemSelection &delSel)
{
	// FIXME Currently this works only from treeview->ME
	treeview->expandAll();	//FIXME only for testing

	//((VymModel*)treeview->model())->select ();
}

